==> ircii-2.8.36-pana.tar.gz: ircii-2.8/source/notify.c <==
/*
 * notify.c: a few handy routines to notify you when people enter and leave irc 
 *
 * Written By Michael Sandrof
 * Copyright(c) 1990 
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 *
 * Revamped by lynX - Dec '91
 */


#include "irc.h"

#include "list.h"
#include "notify.h"
#include "ircaux.h"
#include "whois.h"
#include "hook.h"
#include "server.h"
#include "output.h"
#include "vars.h"
#include "timer.h"
#include "pluto.h"

/* NotifyList: the structure for the notify stuff */
typedef	struct	notify_stru
{
	struct	notify_stru	*next;	/* pointer to next notify person */
	char	*nick;			/* nickname of person to notify about */
	char	*user;
	char	*host;
	int	flag;			/* 1=person on irc, 0=person not on irc */
}	NotifyList;

extern Server *server_list;
extern int number_of_servers;

extern int no_hook_notify;

static int first_notify = 1;

/* set this on notify to send a whois nick in order to get there host info */

int delay_notify _((void *list))
{
	char *p = (char *)list;
	if (is_server_connected(from_server))
		if (get_int_var(HARD_UH_NOTIFY_VAR))    
			add_to_userhost_queue(p, got_userhost, "%s", " ");
		else
			add_ison_to_whois(p, ison_notify);
	first_notify = 0;
	new_free(&p);
	return 0;
}

/* Rewritten, -lynx */
void
show_notify_list(all)
	int	all;
{
	NotifyList	*tmp;
	char	*list = NULL;

	malloc_strcpy(&list, empty_string);
	
	for (tmp = server_list[from_server].notify_list; tmp; tmp = tmp->next)
	{
		if (tmp->flag)
		{
			malloc_strcat(&list, " ");
			malloc_strcat(&list, tmp->nick);
		}
	}
	
	if (*list)
		say("Currently present:%s", list);
	
	if (all)
	{
		malloc_strcpy(&list, empty_string);
		for (tmp = server_list[from_server].notify_list; tmp; tmp = tmp->next)
		{
			if (!(tmp->flag))
			{
				malloc_strcat(&list, " ");
				malloc_strcat(&list, tmp->nick);
			}
		}
		if (*list) say("Currently absent:%s", list);
	}
	new_free(&list);
}

int hard_uh_notify(int flag, char *nick, char *user, char *host)
{
int servnum = from_server;
NotifyList *new;
char *user1 = NULL, *host1 = NULL;
	if (user && strcmp(user, "<UNKNOWN>"))
		user1 = user;
	if (host && strcmp(host, "<UNKNOWN>"))
		host1 = host;
	for (servnum = 0; servnum < number_of_servers; servnum++)
	{
		if (!(new = (NotifyList *) find_in_list((List **)&server_list[servnum].notify_list, nick, 0)))
		{
			new = (NotifyList *) new_malloc(sizeof(NotifyList));
			memset(new, 0, sizeof(NotifyList));
			new->nick = m_strdup(nick);
			new->user = m_strdup(user1);
			new->host = m_strdup(host1);
			new->flag = 0;
			add_to_list((List **)&server_list[servnum].notify_list, (List *) new);
			return 1;		
		}
		if (!new->user || (user1 && strcmp(new->user, user1)))
			malloc_strcpy(&new->user, user1);
		if (!new->host || (user1 && strcmp(new->host, host1)))
			malloc_strcpy(&new->host, host1);
	}
	return 1;
}

/* notify: the NOTIFY command.  Does the whole ball-o-wax */
/*ARGSUSED*/
void
notify(command, args, subargs)
	char	*command,
		*args,
		*subargs;
{
	char	*nick,
		*list = NULL,
		*ptr;
	int	no_nicks = 1;
	int	do_ison = 0;
	int	servnum = from_server;
#ifdef EPIC_LAMER
	int	old_from_server = from_server;
#endif
	int	shown = 0;
	NotifyList	*new;

	malloc_strcpy(&list, empty_string);
	while ((nick = next_arg(args, &args)) != NULL)
	{
		no_nicks = 0;
		while (nick)
		{
			if ((ptr = index(nick, ',')) != NULL)
				*ptr++ = '\0';
			
			if (*nick == '-')
			{
				nick++;
				
				if (*nick)
				{
					for (servnum = 0; servnum < number_of_servers; servnum++)
					{
						if ((new = (NotifyList *) remove_from_list((List **)&(server_list[servnum].notify_list), nick)))
						{
							new_free(&(new->nick));
							new_free(&(new->host));
							new_free(&(new->user));
							new_free((char **)&new);
							
							if (!shown)
							{
								bitchsay("%s removed from notification list", nick);
								shown = 1;
							}
						}
						else
						{
							if (!shown)
							{
								bitchsay("%s is not on the notification list", nick);
								shown = 1;
							}
						}
					}
				}
				else
				{
					for (servnum = 0; servnum < number_of_servers; servnum++)
					{
						while ((new = server_list[servnum].notify_list))
						{
							server_list[servnum].notify_list = new->next;
							new_free(&new->nick);
							new_free(&(new->user));
							new_free(&new->host);
							new_free((char **)&new);
						}
					}
					bitchsay("Notify list cleared");
				}
			}
			else
			{
				/* compatibility */
				if (*nick == '+')
					nick++;

				if (*nick)
				{
					if (index(nick, '*'))
						bitchsay("Wildcards not allowed in NOTIFY nicknames!");
					else
					{
						do_ison = 1;
						for (servnum = 0; servnum < number_of_servers; servnum++)
						{
							if ((new = (NotifyList *) remove_from_list((List **)&server_list[servnum].notify_list, nick)) != NULL)
							{
								new_free(&(new->nick));
								new_free(&(new->user));
								new_free(&(new->host));
								new_free((char **)&new);
							}
							new = (NotifyList *) new_malloc(sizeof(NotifyList));
							memset(new, 0, sizeof(NotifyList));
							new->nick = m_strdup(nick);
							new->flag = 0;
							add_to_list((List **)&server_list[servnum].notify_list, (List *) new);
						}
						malloc_strcat(&list, new->nick);
						malloc_strcat(&list, " ");
						bitchsay("%s added to the notification list", nick);
					}
				} else
					show_notify_list(0);
			}
			nick = ptr;
		}
	}

	if (do_ison)
	{
		if (first_notify)
			add_timer(-1, 10, delay_notify, list, NULL);
		else
		{
#ifdef EPIC_LAMER
			for (servnum = 0; servnum < number_of_servers; servnum++)
			{
				from_server = servnum;
#endif	
				if (is_server_connected(from_server))
					if (get_int_var(HARD_UH_NOTIFY_VAR))
						add_to_userhost_queue(list, got_userhost, "%s", list);
					else
						add_ison_to_whois(list, ison_notify);
#ifdef EPIC_LAMER
			}
			from_server = old_from_server;
#endif
		}
	}
	
	if (!first_notify)
		new_free(&list);	
	if (no_nicks)
		show_notify_list(1);
}

/*
 * do_notify: This simply goes through the notify list, sending out a WHOIS
 * for each person on it.  This uses the fancy whois stuff in whois.c to
 * figure things out.  Look there for more details, if you can figure it out.
 * I wrote it and I can't figure it out.
 *
 * Thank you Michael... leaving me bugs to fix :) Well I fixed them!
 */
void
do_notify()
{
	int	size;
	int	old_from_server = from_server;
	int	servnum = from_server;
	char	buf[BIG_BUFFER_SIZE+1];
	NotifyList	*tmp;

	if (!number_of_servers)
		return;


	tmp = server_list[0].notify_list;
	while (tmp)
	{
		for (*buf = 0, size = 0; tmp; tmp = tmp->next)
		{
			size ++; /*= strlen(tmp->nick) + 1;*/
			if (size < 5)
			{
				strcat(buf, " ");
				strcat(buf, tmp->nick);
			}
			else
				break;
		}

		if (!size)
			continue;
#ifdef EPIC_LAME
		for (servnum = 0; servnum < number_of_servers; servnum++)
		{
#endif
			if (is_server_connected(servnum))
			{
				from_server = servnum;
				if (get_int_var(HARD_UH_NOTIFY_VAR))
					add_to_userhost_queue(buf, got_userhost, "%s", buf);
				else
					add_ison_to_whois(buf, ison_notify);
			}
#ifdef EPIC_LAME
		}
#endif
	}
	from_server = old_from_server;
	return;
}

void check_auto_invite(char *nick, char *user, char *host)
{
char *channel;
ChannelList *tmp_chan;
char *uh = NULL;
NickList *tmp;
	channel = get_channel_by_refnum(0);
	malloc_sprintf(&uh, "%s@%s", user, host);
	if (channel && ((tmp = nickinlist(user_list, nick, uh)) && wild_match(tmp->channels, channel)))
	{
		tmp_chan = (ChannelList *)find_in_list((List **)&(server_list[from_server].chan_list), channel, 0);
		if (tmp_chan && tmp_chan->set_ainv && tmp->userlevel >=tmp_chan->set_ainv)
		{
			bitchsay("Auto-inviting %s to %s", nick, channel);
			send_to_server("NOTICE %s :Auto-invite from %s", nick, get_server_nickname(from_server));
			send_to_server("INVITE %s %s", nick, channel);
		}
	}
	new_free(&uh);
}

/*
 * notify_mark: This marks a given person on the notify list as either on irc
 * (if flag is 1), or not on irc (if flag is 0).  If the person's status has
 * changed since the last check, a message is displayed to that effect.  If
 * the person is not on the notify list, this call is ignored 
 * doit if passed as 0 means it comes from a join, or a msg, etc, not from
 * an ison reply.  1 is the other..
 */
void
notify_mark(nick, flag, doit)
	char	*nick;
	int	flag;
	int	doit;
{
	NotifyList	*tmp;
	char	*s = get_string_var(NOTIFY_HANDLER_VAR);

	if (!s || (!doit && ('O' == *s)))	/* old notify */
		return;	
		
	if ('N' == *s)			/* noisy notify */
		doit = 1;

	if ((tmp = (NotifyList *) list_lookup((List **) &server_list[from_server].notify_list, nick, !USE_WILDCARDS, !REMOVE_FROM_LIST)) != NULL)
	{
		if (flag)
		{
			if (tmp->flag != 1)
			{
				if (get_int_var(HARD_UH_NOTIFY_VAR))
				{
					if ((tmp->flag != -1) && (do_hook(NOTIFY_SIGNON_UH_LIST, "%s %s %s", nick, tmp->user?tmp->user:"", tmp->host?tmp->host:"")) && doit)
						bitchsay("Signon by %s!%s@%s detected", nick, tmp->user?tmp->user:"", tmp->host);
					if (tmp->flag != -1 && doit)
						check_auto_invite(nick, tmp->user, tmp->host);
					
				}
				else
					if ((tmp->flag != -1) && (do_hook(NOTIFY_SIGNON_LIST, "%s", nick)) && doit)
						bitchsay("Signon by %s detected", nick);
				/*
				 * copy the correct case of the nick
				 * into our array  ;)
				 */
				malloc_strcpy(&(tmp->nick), nick);
				malloc_strcpy(&last_notify_nick, nick);
				tmp->flag = 1;
			}
		}
		else
		{
			if (get_int_var(HARD_UH_NOTIFY_VAR))
				if ((tmp->flag == 1) && (do_hook(NOTIFY_SIGNOFF_UH_LIST, "%s %s %s", nick, tmp->user?tmp->user:"", tmp->host?tmp->host:"")) && doit)
					bitchsay("Signoff by %s!%s@%s detected", nick, tmp->user?tmp->user:"",tmp->host);
			else
				if ((tmp->flag == 1) && (do_hook(NOTIFY_SIGNOFF_LIST, "%s", nick)) && doit)
					bitchsay("Signoff by %s detected", nick);
			tmp->flag = 0;
		}
	}
}

void
save_notify(fp)
	FILE	*fp;
{
	int size = 0;
	int count = 0;
	NotifyList	*tmp;

	if (number_of_servers && server_list[0].notify_list)
	{
		fprintf(fp, "NOTIFY");
		for (tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
		{
			size++; /* += (strlen(tmp->nick) + 1);*/
			fprintf(fp, " %s", tmp->nick);
			if (size >= 5)
			{
				fprintf(fp, "\n");
				fprintf(fp, "NOTIFY");
				size = 0;
			}
			count ++;
		}
		fprintf(fp, "\n");
	}
	bitchsay("Saved %d Notify entries", count);
}

/* I hate broken compilers -mrg */
static	char	*vals[] = { "NOISY", "QUIET", "OLD", NULL };

void
set_notify_handler(value)
	char	*value;
{
	int	len;
	int	i;
	char	*s;

	if (!value)
		value = empty_string;
	for (i = 0, len = strlen(value); (s = vals[i]); i++)
		if (0 == my_strnicmp(value, s, len))
			break;
	set_string_var(NOTIFY_HANDLER_VAR, s);
	return;
}

#ifdef __STDC__
void make_notify_list (int servnum)
#else
void make_notify_list (servnum)
	int servnum;
#endif
{
	NotifyList *tmp;
	char *list = NULL;

	server_list[servnum].notify_list = NULL;

	for (tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
	{
		NotifyList *new = (NotifyList *)new_malloc(sizeof(NotifyList));
		new->nick = m_strdup(tmp->nick);
		new->host = m_strdup(tmp->host);
		new->user = m_strdup(tmp->user);
		new->flag = 0;
		add_to_list((List **)&server_list[servnum].notify_list, (List *)new);
		malloc_strcat(&list, new->nick);
		malloc_strcat(&list, " ");

	}

	if (get_int_var(HARD_UH_NOTIFY_VAR))
	{
		new_free(&list);
		for ( tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
		{
			if (is_server_connected(servnum))
				add_to_userhost_queue(tmp->nick, got_userhost, "%s", tmp->nick);
		}
	}
	else
	{
		if (list)
		{
			if (is_server_connected(servnum))
				add_ison_to_whois(list, ison_notify);
			new_free(&list);
		}
	}

}

==> ircii-2.8.43-pana.tar.gz: ircii-2.8/source/notify.c <==
/*
 * notify.c: a few handy routines to notify you when people enter and leave irc 
 *
 * Written By Michael Sandrof
 * Copyright(c) 1990 
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 *
 * Revamped by lynX - Dec '91
 */


#include "irc.h"

#include "list.h"
#include "notify.h"
#include "ircaux.h"
#include "whois.h"
#include "hook.h"
#include "server.h"
#include "output.h"
#include "vars.h"
#include "timer.h"
#include "pluto.h"
#include "userlist.h"

/* NotifyList: the structure for the notify stuff */
typedef	struct	notify_stru
{
	struct	notify_stru	*next;	/* pointer to next notify person */
	char	*nick;			/* nickname of person to notify about */
	char	*user;
	char	*host;
	int	flag;			/* 1=person on irc, 0=person not on irc */
}	NotifyList;

extern Server *server_list;
extern int number_of_servers;

extern int no_hook_notify;

static int first_notify = 1;

/* set this on notify to send a whois nick in order to get there host info */

int delay_notify _((void *list))
{
	char *p = (char *)list;
	if (is_server_connected(from_server))
		if (get_int_var(HARD_UH_NOTIFY_VAR))    
			add_to_userhost_queue(p, got_userhost, "%s", p);
		else
			add_ison_to_whois(p, ison_notify);
	first_notify = 0;
	new_free(&p);
	return 0;
}

/* Rewritten, -lynx */
void
show_notify_list(all)
	int	all;
{
	NotifyList	*tmp;
	char	*list = NULL;

	malloc_strcpy(&list, empty_string);
	
	for (tmp = server_list[from_server].notify_list; tmp; tmp = tmp->next)
	{
		if (tmp->flag)
		{
			malloc_strcat(&list, " ");
			malloc_strcat(&list, tmp->nick);
		}
	}
	
	if (*list)
		say("Currently present:%s", list);
	
	if (all)
	{
		malloc_strcpy(&list, empty_string);
		for (tmp = server_list[from_server].notify_list; tmp; tmp = tmp->next)
		{
			if (!(tmp->flag))
			{
				malloc_strcat(&list, " ");
				malloc_strcat(&list, tmp->nick);
			}
		}
		if (*list) say("Currently absent:%s", list);
	}
	new_free(&list);
}

int hard_uh_notify(int flag, char *nick, char *user, char *host)
{
int servnum = from_server;
NotifyList *new;
char *user1 = NULL, *host1 = NULL;
	if (user && strcmp(user, "<UNKNOWN>"))
		user1 = user;
	if (host && strcmp(host, "<UNKNOWN>"))
		host1 = host;
	if (!nick)
		return 1;
	for (servnum = 0; servnum < number_of_servers; servnum++)
	{
		if (!(new = (NotifyList *) find_in_list((List **)&server_list[servnum].notify_list, nick, 0)))
		{
			new = (NotifyList *) new_malloc(sizeof(NotifyList));
			memset(new, 0, sizeof(NotifyList));
			new->nick = m_strdup(nick);
			new->user = m_strdup(user1);
			new->host = m_strdup(host1);
			new->flag = 0;
			add_to_list((List **)&server_list[servnum].notify_list, (List *) new);
			return 1;		
		}
		if (!new->user || (user1 && strcmp(new->user, user1)))
			malloc_strcpy(&new->user, user1);
		if (!new->host || (user1 && strcmp(new->host, host1)))
			malloc_strcpy(&new->host, host1);
	}
	return 1;
}

/* notify: the NOTIFY command.  Does the whole ball-o-wax */
/*ARGSUSED*/
void
notify(command, args, subargs)
	char	*command,
		*args,
		*subargs;
{
	char	*nick,
		*list = NULL,
		*ptr;
	int	no_nicks = 1;
	int	do_ison = 0;
	int	servnum = from_server;
#ifdef EPIC_LAMER
	int	old_from_server = from_server;
#endif
	int	shown = 0;
	NotifyList	*new;

	malloc_strcpy(&list, empty_string);
	while ((nick = next_arg(args, &args)) != NULL)
	{
		no_nicks = 0;
		while (nick)
		{
			if ((ptr = index(nick, ',')) != NULL)
				*ptr++ = '\0';
			
			if (*nick == '-')
			{
				nick++;
				
				if (*nick)
				{
					for (servnum = 0; servnum < number_of_servers; servnum++)
					{
						if ((new = (NotifyList *) remove_from_list((List **)&(server_list[servnum].notify_list), nick)))
						{
							new_free(&(new->nick));
							new_free(&(new->host));
							new_free(&(new->user));
							new_free((char **)&new);
							
							if (!shown)
							{
								bitchsay("%s removed from notification list", nick);
								shown = 1;
							}
						}
						else
						{
							if (!shown)
							{
								bitchsay("%s is not on the notification list", nick);
								shown = 1;
							}
						}
					}
				}
				else
				{
					for (servnum = 0; servnum < number_of_servers; servnum++)
					{
						while ((new = server_list[servnum].notify_list))
						{
							server_list[servnum].notify_list = new->next;
							new_free(&new->nick);
							new_free(&(new->user));
							new_free(&new->host);
							new_free((char **)&new);
						}
					}
					bitchsay("Notify list cleared");
				}
			}
			else
			{
				/* compatibility */
				if (*nick == '+')
					nick++;

				if (*nick)
				{
					if (index(nick, '*'))
						bitchsay("Wildcards not allowed in NOTIFY nicknames!");
					else
					{
						do_ison = 1;
						for (servnum = 0; servnum < number_of_servers; servnum++)
						{
							if ((new = (NotifyList *) remove_from_list((List **)&server_list[servnum].notify_list, nick)) != NULL)
							{
								new_free(&(new->nick));
								new_free(&(new->user));
								new_free(&(new->host));
								new_free((char **)&new);
							}
							new = (NotifyList *) new_malloc(sizeof(NotifyList));
							memset(new, 0, sizeof(NotifyList));
							new->nick = m_strdup(nick);
							new->flag = 0;
							add_to_list((List **)&server_list[servnum].notify_list, (List *) new);
						}
						malloc_strcat(&list, new->nick);
						malloc_strcat(&list, " ");
						bitchsay("%s added to the notification list", nick);
					}
				} else
					show_notify_list(0);
			}
			nick = ptr;
		}
	}

	if (do_ison)
	{
		if (first_notify)
			add_timer(-1, 10, delay_notify, list, NULL);
		else
		{
#ifdef EPIC_LAMER
			for (servnum = 0; servnum < number_of_servers; servnum++)
			{
				from_server = servnum;
#endif	
				if (is_server_connected(from_server))
					if (get_int_var(HARD_UH_NOTIFY_VAR))
						add_to_userhost_queue(list, got_userhost, "%s", list);
					else
						add_ison_to_whois(list, ison_notify);
#ifdef EPIC_LAMER
			}
			from_server = old_from_server;
#endif
		}
	}
	
	if (!first_notify)
		new_free(&list);	
	if (no_nicks)
		show_notify_list(1);
}

/*
 * do_notify: This simply goes through the notify list, sending out a WHOIS
 * for each person on it.  This uses the fancy whois stuff in whois.c to
 * figure things out.  Look there for more details, if you can figure it out.
 * I wrote it and I can't figure it out.
 *
 * Thank you Michael... leaving me bugs to fix :) Well I fixed them!
 */
void
do_notify()
{
	int	size;
	int	old_from_server = from_server;
	int	servnum = from_server;
	char	buf[BIG_BUFFER_SIZE+1];
	NotifyList	*tmp;

	if (!number_of_servers)
		return;


	tmp = server_list[0].notify_list;
	while (tmp)
	{
		for (*buf = 0, size = 0; tmp; tmp = tmp->next)
		{
			size ++; /*= strlen(tmp->nick) + 1;*/
			if (size < 5)
			{
				strcat(buf, " ");
				strcat(buf, tmp->nick);
			}
			else
				break;
		}

		if (!size)
			continue;
#ifdef EPIC_LAME
		for (servnum = 0; servnum < number_of_servers; servnum++)
		{
#endif
			if (is_server_connected(servnum))
			{
				from_server = servnum;
				if (get_int_var(HARD_UH_NOTIFY_VAR))
					add_to_userhost_queue(buf, got_userhost, "%s", buf);
				else
					add_ison_to_whois(buf, ison_notify);
			}
#ifdef EPIC_LAME
		}
#endif
	}
	from_server = old_from_server;
	return;
}

void check_auto_invite(char *nick, char *user, char *host)
{
char *channel;
ChannelList *tmp_chan;
char *uh = NULL;
NickList *tmp;
	channel = get_channel_by_refnum(0);
	malloc_sprintf(&uh, "%s@%s", user, host);
	if (channel && ((tmp = nickinlist(user_list, nick, uh)) && check_channel_match(tmp->channels, channel)))
	{
		tmp_chan = (ChannelList *)find_in_list((List **)&(server_list[from_server].chan_list), channel, 0);
		if (tmp_chan && tmp_chan->set_ainv && tmp->userlevel >=tmp_chan->set_ainv)
		{
			bitchsay("Auto-inviting %s to %s", nick, channel);
			send_to_server("NOTICE %s :Auto-invite from %s", nick, get_server_nickname(from_server));
			send_to_server("INVITE %s %s", nick, channel);
		}
	}
	new_free(&uh);
}

/*
 * notify_mark: This marks a given person on the notify list as either on irc
 * (if flag is 1), or not on irc (if flag is 0).  If the person's status has
 * changed since the last check, a message is displayed to that effect.  If
 * the person is not on the notify list, this call is ignored 
 * doit if passed as 0 means it comes from a join, or a msg, etc, not from
 * an ison reply.  1 is the other..
 */
void
notify_mark(nick, flag, doit)
	char	*nick;
	int	flag;
	int	doit;
{
	NotifyList	*tmp;
	char	*s = get_string_var(NOTIFY_HANDLER_VAR);

	if (!s || (!doit && ('O' == *s)))	/* old notify */
		return;	
		
	if ('N' == *s)			/* noisy notify */
		doit = 1;
	if (!nick)
		return;
	if ((tmp = (NotifyList *) list_lookup((List **) &server_list[from_server].notify_list, nick, !USE_WILDCARDS, !REMOVE_FROM_LIST)) != NULL)
	{
		if (flag)
		{
			if (tmp->flag != 1)
			{
				if (get_int_var(HARD_UH_NOTIFY_VAR))
				{
					if ((tmp->flag != -1) && (do_hook(NOTIFY_SIGNON_UH_LIST, "%s %s %s", nick, tmp->user?tmp->user:"", tmp->host?tmp->host:"")) && doit)
						bitchsay("Signon by %s!%s@%s detected", nick, tmp->user?tmp->user:"", tmp->host);
					if (tmp->flag != -1 && doit)
						check_auto_invite(nick, tmp->user, tmp->host);
					
				}
				else
					if ((tmp->flag != -1) && (do_hook(NOTIFY_SIGNON_LIST, "%s", nick)) && doit)
						bitchsay("Signon by %s detected", nick);
				/*
				 * copy the correct case of the nick
				 * into our array  ;)
				 */
				malloc_strcpy(&(tmp->nick), nick);
				malloc_strcpy(&last_notify_nick, nick);
				tmp->flag = 1;
			}
		}
		else
		{
			if (get_int_var(HARD_UH_NOTIFY_VAR))
				if ((tmp->flag == 1) && (do_hook(NOTIFY_SIGNOFF_UH_LIST, "%s %s %s", nick, tmp->user?tmp->user:"", tmp->host?tmp->host:"")) && doit)
					bitchsay("Signoff by %s!%s@%s detected", nick, tmp->user?tmp->user:"",tmp->host);
			else
				if ((tmp->flag == 1) && (do_hook(NOTIFY_SIGNOFF_LIST, "%s", nick)) && doit)
					bitchsay("Signoff by %s detected", nick);
			tmp->flag = 0;
		}
	}
}

void
save_notify(fp)
	FILE	*fp;
{
	int size = 0;
	int count = 0;
	NotifyList	*tmp;

	if (number_of_servers && server_list[0].notify_list)
	{
		fprintf(fp, "NOTIFY");
		for (tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
		{
			size++; /* += (strlen(tmp->nick) + 1);*/
			fprintf(fp, " %s", tmp->nick);
			if (size >= 5)
			{
				fprintf(fp, "\n");
				fprintf(fp, "NOTIFY");
				size = 0;
			}
			count ++;
		}
		fprintf(fp, "\n");
	}
	if (do_hook(SAVEFILE_LIST, "Notify %d", count))
		bitchsay("Saved %d Notify entries", count);
}

/* I hate broken compilers -mrg */
static	char	*vals[] = { "NOISY", "QUIET", "OLD", NULL };

void
set_notify_handler(value)
	char	*value;
{
	int	len;
	int	i;
	char	*s;

	if (!value)
		value = empty_string;
	for (i = 0, len = strlen(value); (s = vals[i]); i++)
		if (0 == my_strnicmp(value, s, len))
			break;
	set_string_var(NOTIFY_HANDLER_VAR, s);
	return;
}

#ifdef __STDC__
void make_notify_list (int servnum)
#else
void make_notify_list (servnum)
	int servnum;
#endif
{
	NotifyList *tmp;
	char *list = NULL;

	server_list[servnum].notify_list = NULL;

	for (tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
	{
		NotifyList *new = (NotifyList *)new_malloc(sizeof(NotifyList));
		new->nick = m_strdup(tmp->nick);
		new->host = m_strdup(tmp->host);
		new->user = m_strdup(tmp->user);
		new->flag = 0;
		add_to_list((List **)&server_list[servnum].notify_list, (List *)new);
		malloc_strcat(&list, new->nick);
		malloc_strcat(&list, " ");

	}

	if (get_int_var(HARD_UH_NOTIFY_VAR))
	{
		new_free(&list);
		for ( tmp = server_list[0].notify_list; tmp; tmp = tmp->next)
		{
			if (is_server_connected(servnum))
				add_to_userhost_queue(tmp->nick, got_userhost, "%s", tmp->nick);
		}
	}
	else
	{
		if (list)
		{
			if (is_server_connected(servnum))
				add_ison_to_whois(list, ison_notify);
			new_free(&list);
		}
	}

}

char *get_notify_nicks (int showserver, int showon)
{
	NotifyList *tmp;
	char *list = NULL;

	if (showserver >= number_of_servers)
		return m_strdup(empty_string);
 
	for (tmp = server_list[showserver].notify_list; tmp; tmp = tmp->next)
	{
		if (showon == -1 || showon == tmp->flag)
			m_s3cat(&list, " ", tmp->nick);
	}

	return list ? list : m_strdup(empty_string);
}
